@@ -70,7 +70,7 @@ class IterableIPShell:
     '''
     io = IPython.utils.io
     if input_func:
-      IPython.frontend.terminal.interactiveshell.raw_input_original = input_func
+      IPython.terminal.interactiveshell.raw_input_original = input_func
     if cin:
       io.stdin = io.IOStream(cin)
     if cout:
@@ -98,7 +98,7 @@ class IterableIPShell:
 
     # InteractiveShell inherits from SingletonConfigurable, so use instance()
     #
-    self.IP = IPython.frontend.terminal.embed.InteractiveShellEmbed.instance(\
+    self.IP = IPython.terminal.embed.InteractiveShellEmbed.instance(\
             config=cfg, user_ns=user_ns)
 
     sys.stdout, sys.stderr = old_stdout, old_stderr
@@ -169,11 +169,13 @@ class IterableIPShell:
           self.IP.autoedit_syntax):
           self.IP.edit_syntax_error()
       if not self.iter_more:
-          source_raw = self.IP.input_splitter.source_raw_reset()[1]
+          source_raw = self.IP.input_splitter.raw_reset()
           self.IP.run_cell(source_raw, store_history=True)
+          self.IP.rl_do_indent = False
       else:
           # TODO: Auto-indent
           #
+          self.IP.rl_do_indent = True
           pass
 
     sys.stdout = orig_stdout
@@ -462,6 +464,10 @@ class ConsoleView(gtk.TextView):
     self.text_buffer.move_mark(self.line_start, self.text_buffer.get_end_iter())
     self.text_buffer.place_cursor(self.text_buffer.get_end_iter())
 
+    if self.IP.rl_do_indent:
+      indentation = self.IP.input_splitter.indent_spaces * ' '
+      self.text_buffer.insert_at_cursor(indentation)
+
   def onKeyPress(self, widget, event):
     '''
     Key press callback used for correcting behavior for console-like 
@@ -600,7 +606,8 @@ class IPythonView(ConsoleView, IterableIPShell):
     if rv: rv = rv.strip('\n')
     self.showReturned(rv)
     self.cout.truncate(0)
-    
+    self.cout.seek(0)
+ 
 if __name__ == "__main__":
   window = gtk.Window()
   window.set_default_size(640, 320)
@@ -608,4 +615,4 @@ if __name__ == "__main__":
   window.add(IPythonView())
   window.show_all()
   gtk.main()
-    
+ 
